head	1.2;
access;
symbols;
locks; strict;
comment	@// @;


1.2
date	96.10.08.19.47.12;	author idr;	state Exp;
branches;
next	1.1;

1.1
date	96.10.07.22.24.59;	author idr;	state Exp;
branches;
next	;


desc
@@


1.2
log
@Move the FixMove() function to board.cc.  I also added the ability for
one of the player programs to be specified as "-".  If this is specified,
then the player is not a program, but a human.
@
text
@/* Name : quixo.cc
 *
 * Notes:
 *
 * $Log: quixo.cc $
 * Revision 1.1  1996/10/07 22:24:59  idr
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdtypes.h>
#include "board.h"
#include "quixo.h"

void ShowUsage( const char * name );
void StartProgram( const char * program, FILE * fd[], const char * player );
void FixMove( move_t * m, int x, int y, int offset, char direction );
void InitPlayer( struct Player * p, const char * name, int playerNumber );

/*****************************************************************************/
/* Name : main()
 *
 * Notes:
 *
 */

int main( int argc, char ** argv )
{
    struct Player   player1;        /* Data for talking to player 1.        */
    struct Player   player2;        /* Data for talking to player 2.        */
    int             winner;         /* Who won the game?                    */
    Board           b;              /* Current game board.                  */
    move_t          m;              /* Last move that was made.             */
    int             totalMoves;     /* Total moves made in the game.        */



    if ( argc < 3 )
    {
        ShowUsage( argv[ 0 ] );
        exit( 1 );
    }


    /* Initialize the display in the input stuff. */

    OpenDisplay();
    InitInput();


    /* Create the structures for talking to our two players. */

    InitPlayer( & player1, argv[ 1 ], PLAYER_1 );
    InitPlayer( & player2, argv[ 2 ], PLAYER_2 );


    totalMoves = 0;
    ShowBoard( b );

    /* Play the game until somebody wins, or until they hit ^C. :) */

    while ( 1 )
    {
        /* Get the first player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 2
         * as the winner.
         */

        m = player1.getMove( b, & player1 );

        totalMoves++;
        if ( ! b.DoMove( m, PLAYER_1 ) )
        {
            DisplayMove( & m, NULL );
            winner = PLAYER_2;
            break;
	}

        ShowBoard( b );

        /* It that move caused somebody to win the game, then we'll need to
         * break out of the loop.  The game is over.
         */

        winner = b.IsWinner( PLAYER_1 );
        if ( winner != EMPTY ) break;


        /* Tell player 2 *exacly* what player 1 did. */

        player2.showMove( & m, & player2 );



        /* Get the second player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 1
         * as the winner.
         */

        m = player2.getMove( b, & player2 );

        totalMoves++;
        if ( ! b.DoMove( m, PLAYER_2 ) )
        {
            winner = PLAYER_1;
            break;
	}

        ShowBoard( b );

        /* It that move caused somebody to win the game, then we'll need to
         * break out of the loop.  The game is over.
         */

        winner = b.IsWinner( PLAYER_2 );
        if ( winner != EMPTY ) break;


        /* Tell player 1 *exacly* what player 2 did. */

        player1.showMove( & m, & player1 );
    }


    /* Tell them who won, close up shop, and terminate. */

    DisplayWinner( winner, totalMoves );
    CloseDisplay();
    return( 0 );
}




/*****************************************************************************/
/* Name : InitPlayer()
 *
 * Notes: This function initializes the player structures.  The the name of
 *        the "file" assosicated with the player is "-", then the player is
 *        actually a human player.  If not, then we'll need to start the
 *        specified program to play as this player.
 */

void InitPlayer( struct Player * p, const char * name, int playerNumber )
{
    p->player = playerNumber;

    if ( strcmp( "-", name ) == 0 )
    {
        p->getMove = GetHumanMove;
        p->showMove = DisplayMove;
    }
    else
    {
        p->getMove = GetComputerMove;
        p->showMove = PutComputerMove;
        StartProgram( name, p->fd, (playerNumber == -1) ? "1" : "2" );
    }
}




/*****************************************************************************/
/* Name : StartProgram()
 *
 * Notes: This function starts a new process and runs the named program.  It
 *        fills fd[0] with a file handle that is tied to the new program's
 *        STDOUT (i.e., you can read the output of the child program through
 *        this file), and fills fd[1] with a file handle that is tied to the
 *        new program's STDIN.
 */

void StartProgram( const char * program, FILE * fd[], const char * player )
{
    int     pipeIn[ 2 ];        /* Provides STDIN for the CHILD.    */
    int     pipeOut[ 2 ];       /* Provides STDOUT for the CHILD.   */


    /* First, create the two pipes that we will use to communicate with the
     * child processes.
     */

    if ( pipe( pipeIn ) == -1 )
    {
        perror( "pipe( pipeIn )" );
        exit( 1 );
    }

    if ( pipe( pipeOut ) == -1 )
    {
        perror( "pipe( pipeOut )" );
        exit( 1 );
    }



    /* Create the child process! */

    if ( fork() == 0 )
    {
        /* Close the default STDIN and STDOUT file handles.  We want our
         * STDIN and STDOUT tied directly to our parent process.
         */

        close( STDIN_FILENO );
        close( STDOUT_FILENO );


        /* Close the write part of our STDIN pipe and the read part of our
         * STDOUT pipe.  As the child, we can't use these.
         */

        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );



        if ( dup2( pipeIn[ 0 ], STDIN_FILENO ) == -1 )
        {
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
            exit( 1 );
        }

        if ( dup2( pipeOut[ 1 ], STDOUT_FILENO ) == -1 )
        {
            perror( "dup2( pipeOut[ 1 ], STDOUT_FILENO )" );
            exit( 1 );
        }


        /* Run the specified program. */

        execl( program, program, player, NULL );


        /* If we get to this point, it means that execl() failed.  Let's
         * print an error message and exit.
         */

        perror( "execl()" );
        exit( 1 );
    }




    /* Close the read part of the child's STDIN pipe and the out part of
     * the child's STDOUT pipe.  As the parent, we (can't) won't be using
     * these parts.
     */

    close( pipeIn[ 0 ] );
    close( pipeOut[ 1 ] );


    /* Convert the parts of the pipes that we will be using to normal
     * stdio style FILEs.
     */

    fd[ 0 ] = fdopen( pipeOut[ 0 ], "r" );
    if ( fd[ 0 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }

    fd[ 1 ] = fdopen( pipeIn[ 1 ], "w" );
    if ( fd[ 1 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }
}




/*****************************************************************************/
/* Name : ShowUsage()
 *
 * Notes: This function attempts to show the user how to properly run our
 *        little program.
 */

void ShowUsage( const char * name )
{
    fprintf( stderr, "Usage: %s player1_program player2_program\n" \
                     "    If the player_program is \"-\", then the player " \
                     "actually a human player.\n",
             name );
}
@


1.1
log
@Initial revision
@
text
@d5 4
a8 1
 * $Log:$
d16 1
d21 1
d32 6
a37 4
    FILE  * player1[ 2 ];
    FILE  * player2[ 2 ];
    int     winner;
    Board   b;
a38 5
    int     x;
    int     y;
    int     offset;
    char    direction;
    move_t  m;
a46 2
    StartProgram( argv[ 1 ], player1, "1" );
    StartProgram( argv[ 2 ], player2, "2" );
d48 1
d50 15
d67 4
a70 2
        fscanf( player1[ 0 ], "%d %d %d %c", & x, & y, & offset, & direction );
        FixMove( & m, x, y, offset, direction );
d72 3
d77 1
d82 1
d84 4
a90 1
        fprintf( player2[ 1 ], "%d %d %d %c\n", x, y, offset, direction );
d92 1
d94 1
a94 2
        fscanf( player2[ 0 ], "%d %d %d %c", & x, & y, & offset, & direction );
        FixMove( & m, x, y, offset, direction );
d96 10
d111 7
a117 1
        
d121 4
a124 1
        fprintf( player2[ 1 ], "%d %d %d %c\n", x, y, offset, direction );
d128 1
a128 1
    printf( "The winner is player %d.\n", winner );
d130 2
d139 7
d147 30
d182 5
d201 1
d205 4
d212 10
d230 1
a230 1
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
a233 2
        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );
d235 1
a235 1
        execl( NULL, "program", player, NULL );
d237 7
d251 5
d259 5
d272 1
a272 1
    if ( fd[ 0 ] == NULL )
a276 1

d283 5
a288 34
void FixMove( move_t * m, int x, int y, int offset, char direction )
{
    m->srcX = x;
    m->srcY = y;

    switch( direction )
    {
        case 'u':
            m->destX = x;
            m->destY = 4;
            break;

        case 'd':
            m->destX = x;
            m->destY = 0;
            break;

        case 'l':
            m->destX = 4;
            m->destY = y;
            break;

        case 'r':
            m->destX = 0;
            m->destY = y;
            break;
    }
}




/*****************************************************************************/

d291 4
a294 1
    fprintf( stderr, "Usage: %s player1_program player2_program\n" );
@
