#include <stdio.h>
#include <string.h>

/* compute scores */
GetHeuristic(char board[5][5], int *scoreX, int *scoreO)
{
 int x,y,numX,numO;

 /** 20 points per piece */
 *scoreX=0; *scoreO=0;
 for (x=0; x<5; x++)
  for (y=0; y<5; y++)
	if (board[x][y]=='0') (*scoreO)+=20;
	else if (board[x][y]=='X') (*scoreX)+=20;

 /** 3 in a row gets 5 points. 4 in a row gets 15 points */
 for (y=0; y<5; y++)
 {
  for (x=0,numX=0,numO=0; x<5; x++)
    if (board[x][y]=='O') numO++;
    else if (board[x][y]=='X') numX++;
  if (numO==3) (*scoreO)+=5;
  else if (numO==4) (*scoreO)+=15;
  if (numX==3) (*scoreX)+=5;
  else if (numX==4) (*scoreX)+=15;
 }
 for (x=0; x<5; x++)
 {
  for (y=0,numX=0,numO=0; y<5; y++)
    if (board[x][y]=='O') numO++;
    else if (board[x][y]=='X') numX++;
  if (numO==3) (*scoreO)+=5;
  else if (numO==4) (*scoreO)+=15;
  if (numX==3) (*scoreX)+=5;
  else if (numX==4) (*scoreX)+=15;
 }

 /* Check diagonals */
 for (numO=0,numX=0,x=0,y=0; x<5; x++,y++)
 {
  if (board[x][y]=='O') numO++;
  else if (board[x][y]=='X') numX++;
 }
 if (numO==3) (*scoreO)+=5;
 else if (numO==4) (*scoreO)+=15;
 if (numX==3) (*scoreX)+=5;
 else if (numX==4) (*scoreX)+=15;
 for (numO=0,numX=0,x=0,y=4; x<5; x++,y--)
 {
  if (board[x][y]=='O') numO++;
  else if (board[x][y]=='X') numX++;
 }
 if (numO==3) (*scoreO)+=5;
 else if (numO==4) (*scoreO)+=15;
 if (numX==3) (*scoreX)+=5;
 else if (numX==4) (*scoreX)+=15;
}


/* Max part of minimax search w/alpha beta pruning */
int MaxValue(char board[5][5], int currentMove, int alpha, int beta, int depth,
		int *xloc, int *yloc, char *sSide)
{
 int x,y,s,scoreX,scoreO,temp,x1,y2;
 char tempBoard[5][5],s2;
 int opponent;
 int retval;
 char side;

 GetHeuristic(board,&scoreX,&scoreO);
 if (depth==0) 
	{
	  if (currentMove==1) retval=scoreX-scoreO;
	  else retval=scoreO-scoreX;
	}
  else
        { 
          if (currentMove==1) opponent=0; else opponent=1;
          for (x=0; x<5; x++)
	   for (y=0; y<5; y++)
            for (s=0; s<4; s++)
	    {
  		if (s==0) side='l'; else if (s==1) side='r';
		else if (s==2) side='t'; else if (s==3) side='b';
		if (validMove(board,x,y,side,currentMove))
                { 
  		  memcpy((char *) tempBoard[0],(char *) board[0],
			 5*5*sizeof(char));
		  MakeMove(x,y,side,tempBoard,currentMove);
		  temp=CheckForWin(tempBoard,currentMove);
		  if (AddOrCheckBoard(tempBoard,1)==1) temp=-10000;
		  else if (temp==currentMove) temp=10000; 
		  else if (temp>-1) temp=-10000; 
		  else temp=MinValue(tempBoard,opponent,alpha,beta,depth-1,
			 	 &x1,&y2,&s2);
      		  if (temp>alpha) { alpha=temp; *xloc=x; *yloc=y; *sSide=side;}
		  retval=alpha;
                  if (alpha>=beta) { retval=beta; goto leavemax; }
		}
            }
        }
 leavemax:
   return(retval);
}

/* Min part of minimax search w/alpha beta pruning */
int MinValue(char board[5][5], int currentMove, int alpha, int beta, int depth,
		int *xloc, int *yloc, char *sSide)
{
 int x,y,s,scoreX,scoreO,temp;
 char tempBoard[5][5];
 int opponent;
 int retval;
 char side;

 GetHeuristic(board,&scoreX,&scoreO);
 if (depth==0) 
	{
	  if (currentMove==1) retval=scoreX-scoreO;
	  else retval=scoreO-scoreX;
	}
  else
        { 
          if (currentMove==0) opponent=1; else opponent=0;
          for (x=0; x<5; x++)
	   for (y=0; y<5; y++)
	    for (s=0; s<4; s++)
	    {
  		if (s==0) side='l'; else if (s==1) side='r';
		else if (s==2) side='t'; else if (s==3) side='b';
		if (validMove(board,x,y,side,currentMove))
                { 
  		  memcpy((char *) tempBoard[0],(char *) board[0],
			 5*5*sizeof(char));
		  MakeMove(x,y,side,tempBoard,currentMove);
		  temp=CheckForWin(tempBoard,currentMove);
		  if (temp==currentMove) temp=-10000; 
		  else if (temp >-1) temp=10000;
		  else temp=MaxValue(tempBoard,opponent,alpha,beta,depth-1,
			 	 xloc,yloc,sSide);
      		  if (temp<beta) { beta=temp; }
		  retval=beta;
                  if (beta<=alpha) { retval=alpha; goto leavemin; }
		}
            }
        }
 leavemin:
   return(retval);
}

GetComputerMove(char board[5][5], int *x, int *y, char *s, int color, 
		int lookahead)
{
 int best;
 best=MaxValue(board,color,-10000,10000,lookahead,x,y,s);
}

