/* Name : computer.cc
 *
 * Notes: This file contains the functions for the main Quixo(tm) program to
 *        communicate with child programs that represent other players in
 *        the game.
 *
 * $Log: computer.cc $
 * Revision 1.1  1996/10/08 18:03:48  idr
 * Initial revision
 *
 */

#include <stdio.h>
#include "board.h"
#include "quixo.h"

/*****************************************************************************/
/* Name : GetComputerMove()
 *
 * Notes: This function is very much like GetHumanMove(), except that it reads
 *        from a file that is presumed to be the stdout stream for a program
 *        that is playing the game.
 */

move_t GetComputerMove( Board & b, struct Player * p )
{
    move_t  m;
    int     x;
    int     y;
    char    side;

    fscanf( p->fd[0], "%d %d %c", & x, & y, & side );
    FixMove( & m, x, y, side );

    return( m );
}




/*****************************************************************************/
/* Name : PutComputerMove()
 *
 * Notes: This function is used to transmit a move to a specified program.
 *        It is used to send the move made by the other player to the AI
 *        program that is getting ready to make a move.
 */

void PutComputerMove( move_t * m, struct Player * p )
{
    char    side;


    if ( m->srcX == m->destX )
    {
        side = (m->destY == 0) ? 't' : 'b';
    }
    else
    {
        side = (m->destX == 0) ? 'l' : 'r';
    }

    fprintf( p->fd[1], "%d %d %c\n", m->srcX, m->srcY, side );
    fflush( p->fd[1] );
}

