/* Name : display.cc
 *
 * Notes: This file is a contains a skeleton set of display routines.  These
 *        routines are designed to display game information to a "dumb"
 *        terminal.  However, they provide a flexable enough frame work that
 *        GUI versions (i.e., X windows, etc) could be written using the
 *        same interface.
 *
 * $Log: display.cc $
 * Revision 1.2  1996/10/21 15:48:07  idr
 * The DisplayWinner() function was updated to round the number of moves
 * up rather than just truncate.
 *
 * Revision 1.1  1996/10/08 19:48:29  idr
 * Initial revision
 *
 */

#include <stdio.h>
#include "board.h"
#include "quixo.h"

const char playerName[] = { 'X', '_', 'O' };

/*****************************************************************************/
/* Name : OpenDisplay()
 *
 * Notes: This function is used to set up any device specific display stuff.
 *        This would include such things as opening windows and the like.
 */

ERROR OpenDisplay( void )
{
    return( ERROR_NO_ERROR );
}




/******************************************************************************/
/* Name : CloseDisplay()
 *
 * Notes: This function is used to free up any device specific display stuff.
 *        This would include such things as closing windows and the like.
 */

void CloseDisplay( void )
{
    return;
}




/*****************************************************************************/
/* Name : ShowBoard()
 *
 * Notes:
 *
 */

void ShowBoard( Board & b )
{
    int     row;
    int     col;


    fprintf(stdout,"  0 1 2 3 4\n");
    for ( row = 0 ; row < 5 ; row++ )
    {
    	fprintf(stdout,"%d ",row);
        for ( col = 0 ; col < 5 ; col++ )
        {
            fprintf( stdout, "%c ", playerName[ b.board[ col ][ row ] + 1 ] );
        }

        fprintf( stdout,  "\n" );
    }

    fprintf( stdout,  "\n" );
}




/*****************************************************************************/
/* Name : DisplayMovePrompt()
 *
 * Notes: This function is used to let the speicifed player know that it is
 *        time to select a move.
 */

void DisplayMovePrompt( int player )
{
    printf( "Player %c: ", playerName[ player + 1 ] );
    fflush( stdout );
}




/*****************************************************************************/
/* Name : DisplayMove()
 *
 * Notes:
 *
 */

void DisplayMove( move_t * m, struct Player * p )
{
    const char  * side;

    if ( m->srcX == m->destX )
    {
        side = (m->destY == 0) ? "top" : "bottom";
    }
    else
    {
        side = (m->destX == 0) ? "left" : "right";
    }

    printf( "Last move: (%ld, %ld) -> %s\n", m->srcX, m->srcY, side );
    fflush( stdout );
}




/*****************************************************************************/
/* Name : DisplayWinner()
 *
 * Notes:
 *
 */

void DisplayWinner( int player, int moves )
{
    if ( player == EMPTY )
    {
        printf( "The game is a stalemate" );
    }
    else
    {
        printf( "Player %c is the winner", playerName[ player + 1 ] );
    }

    printf( " after %d moves.\n", (moves + 1) / 2 );
    fflush( stdout );
}
