/* Name : dumb.cc
 *
 * Notes: This file contains the code for a 100% dumb Quixo(tm) player.  It
 *        was written purely as an example of how to interface with the
 *        Quixo(tm) game manager.  Feel free to rip any code in this file.
 *
 * $Log: dumb.cc $
 * Revision 1.1  1996/10/08 19:25:56  idr
 * Initial revision
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include "board.h"
#include "quixo.h"

move_t GenerateMove( Board & b, int player );
void SendMove( move_t * m );
void RecvMove( move_t * m );

/*****************************************************************************/

int main( int argc, char ** argv )
{
    Board   b;          /* Current game board.              */
    move_t  m;          /* Last move that was made.         */
    int     us;         /* Which player am I?               */
    int     them;       /* Which player is the opponent?    */



    /* Make sure that we were run properly. */

    if ( argc != 2 )
    {
        fprintf( stderr, "%s: I don't know which player I am.\n", argv[0] );
        exit( 1 );
    }


    /* Set a seed for the random number generator. */

    srand( time( NULL ) );


    /* If I'm the first player, then I need to generate an initial
     * move.
     */

    if ( strcmp( "1", argv[1] ) == 0 )
    {
        us = PLAYER_1;
        them = PLAYER_2;

        m = GenerateMove( b, us );
        b.DoMove( m, us );
        SendMove( & m );
    }
    else
    {
        us = PLAYER_2;
        them = PLAYER_1;
    }


    while( 1 )
    {
        RecvMove( & m );
        b.DoMove( m, them );
        
        m = GenerateMove( b, us );
        b.DoMove( m, us );
        SendMove( & m );
    }
}




/*****************************************************************************/
/* Name : GenerateMove()
 *
 * Notes: This here's the brains of this operation!  I think it might be best
 *        to call this artificial LACK of intelligence.  It basically just
 *        keeps generating RANDOM moves until it finds one that's valid.
 *        Pretty smart, eh?
 */

move_t GenerateMove( Board & b, int player )
{
    move_t  m;


    do
    {
        m.srcX = rand() % 5;
        m.srcY = rand() % 5;

        if ( rand() & 0x01 )
        {
            m.destX = m.srcX;
            m.destY = (rand() & 0x01) ? 0 : 4;
        }
        else
        {
            m.destX = (rand() & 0x01) ? 0 : 4;
            m.destY = m.srcY;
        }

    } while( ! b.IsValidMove( m, player ) );

    return( m );
}




/*****************************************************************************/
/* Name : SendMove()
 *
 * Notes: This function is used to send the specified move to our STDOUT
 *        stream in the standardized Quixo(tm) move format.
 */

void SendMove( move_t * m )
{
    char    side;


//fprintf( stderr, "%d %d %d %d\n", m->srcX, m->srcY, m->destX, m->destY );
    if ( m->srcX == m->destX )
    {
        side = (m->destY == 0) ? 't' : 'b';
    }
    else
    {
        side = (m->destX == 0) ? 'l' : 'r';
    }

    printf( "%d %d %c\n", m->srcX, m->srcY, side );
    fflush( stdout );
}




/*****************************************************************************/
/* Name : RecvMove()
 *
 * Notes: This function is used to read a move from our STDIN stream that is
 *        in the standardized Quixo(tm) move format and convert it to our
 *        internal representation.
 */

void RecvMove( move_t * m )
{
    char    side;


    scanf( "%d %d %c", & m->srcX, & m->srcY, & side );

    FixMove( m, m->srcX, m->srcY, side );
}
