/* Name : quixo.h
 *
 * Notes: This file contains the structure definitions and the function
 *        prototypes used by the Quixo(tm) game manager.
 *
 * $Log: quixo.h $
 * Revision 1.2  1996/10/08 19:53:17  idr
 * I made some minor modifications to the file so that my programs would
 * actually compile.  Oops. :)
 *
 * Revision 1.1  1996/10/08 19:47:12  idr
 * Initial revision
 *
 */

#ifndef QUIXO_H
#define QUIXO_H

#include <stdio.h>

#ifndef STDTYPES_H
#include <stdtypes.h>
#endif

#ifndef ERRORS_H
#include <errors.h>
#endif

/*****************************************************************************/
/* This class is actually defined in the file board.h.
 */

class Board;




/*****************************************************************************/

typedef struct
{
    int srcX;
    int srcY;
    int destX;
    int destY;
} move_t;

/* The time when the most possible moves are available is at the start.  At
 * that time there are three moves for each of the 12 edge pieces, and two
 * moves for each of the corners.
 */

#define MAX_MOVES   ((4*2)+(12*3))




/*****************************************************************************/

struct Player
{
    int     player;     /* Which player number does this represent? */
    FILE  * fd[2];      /* Read from 0, write to 1.                 */

    move_t (*getMove)( Board & b, struct Player * p );
    void   (*showMove)( move_t * m, struct Player * p );
};

#define PLAYER_1    -1
#define PLAYER_2    1
#define EMPTY       0




/*****************************************************************************/

extern void FixMove( move_t * m, int x, int y, char side );

extern ERROR OpenDisplay( void );
extern void CloseDisplay( void );
extern void ShowBoard( Board & b );
extern void DisplayMovePrompt( int player );
extern void DisplayMove( move_t * m, struct Player * p );
extern void DisplayWinner( int player, int moves );

extern ERROR InitInput( void );
extern move_t GetHumanMove( Board & b, struct Player * p );

extern move_t GetComputerMove( Board & b, struct Player * p );
extern void PutComputerMove( move_t * m, struct Player * p );


#endif
